@@ -152,6 +152,14 @@ public:
   SPIRVCapVec getRequiredCapability() const override {
     SPIRVCapVec CV;
     switch (BitWidth) {
+    case 4: {
+      if (Module->isAllowedToUseExtension(ExtensionID::SPV_INTEL_int4)) {
+        CV.push_back(CapabilityInt4TypeINTEL);
+        return CV;
+      }
+      CV.push_back(CapabilityArbitraryPrecisionIntegersINTEL);
+      return CV;
+    }
     case 8:
       CV.push_back(CapabilityInt8);
       break;
@@ -172,6 +180,11 @@ public:
   }
   llvm::Optional<ExtensionID> getRequiredExtension() const override {
     switch (BitWidth) {
+    case 4: {
+      if (Module->isAllowedToUseExtension(ExtensionID::SPV_INTEL_int4))
+        return ExtensionID::SPV_INTEL_int4;
+      return ExtensionID::SPV_INTEL_arbitrary_precision_integers;
+    }
     case 8:
     case 16:
     case 32:
@@ -186,7 +199,9 @@ protected:
   _SPIRV_DEF_ENCDEC3(Id, BitWidth, IsSigned)
   void validate() const override {
     SPIRVEntry::validate();
-    assert((BitWidth == 8 || BitWidth == 16 || BitWidth == 32 ||
+    assert(((BitWidth == 4 &&
+             Module->isAllowedToUseExtension(ExtensionID::SPV_INTEL_int4)) ||
+            BitWidth == 8 || BitWidth == 16 || BitWidth == 32 ||
             BitWidth == 64 ||
             Module->isAllowedToUseExtension(
                 ExtensionID::SPV_INTEL_arbitrary_precision_integers)) &&
@@ -219,6 +234,11 @@ public:
   llvm::Optional<ExtensionID> getRequiredExtension() const override {
     if (isTypeFloat(16, FPEncodingBFloat16KHR))
       return ExtensionID::SPV_KHR_bfloat16;
+    if (isTypeFloat(8, FPEncodingFloat8E4M3EXT) ||
+        isTypeFloat(8, FPEncodingFloat8E5M2EXT))
+      return ExtensionID::SPV_EXT_float8;
+    if (isTypeFloat(4, internal::FPEncodingFloat4E2M1INTEL))
+      return ExtensionID::SPV_INTEL_float4;
     return {};
   }
 
@@ -232,8 +252,14 @@ public:
       if (std::any_of(Extensions.begin(), Extensions.end(),
                       [](const std::string &I) { return I == "cl_khr_fp16"; }))
         CV.push_back(CapabilityFloat16);
-    } else if (isTypeFloat(64))
+    } else if (isTypeFloat(64)) {
       CV.push_back(CapabilityFloat64);
+    } else if (isTypeFloat(8, FPEncodingFloat8E4M3EXT) ||
+               isTypeFloat(8, FPEncodingFloat8E5M2EXT)) {
+      CV.push_back(CapabilityFloat8EXT);
+    } else if (isTypeFloat(4, internal::FPEncodingFloat4E2M1INTEL)) {
+      CV.push_back(internal::CapabilityFloat4E2M1INTEL);
+    }
     return CV;
   }
 
@@ -256,10 +282,16 @@ protected:
 
   void validate() const override {
     SPIRVEntry::validate();
-    assert(BitWidth >= 16 && BitWidth <= 64 && "Invalid bit width");
+    assert((BitWidth == 4 || BitWidth == 8 || BitWidth == 16 ||
+            BitWidth == 32 || BitWidth == 64) &&
+           "Invalid bit width");
     assert(
         (FloatingPointEncoding == FPEncodingMax ||
-         (BitWidth == 16 && FloatingPointEncoding == FPEncodingBFloat16KHR)) &&
+         (BitWidth == 16 && FloatingPointEncoding == FPEncodingBFloat16KHR) ||
+         (BitWidth == 8 && FloatingPointEncoding == FPEncodingFloat8E4M3EXT) ||
+         (BitWidth == 8 && FloatingPointEncoding == FPEncodingFloat8E5M2EXT) ||
+         (BitWidth == 4 &&
+          FloatingPointEncoding == internal::FPEncodingFloat4E2M1INTEL)) &&
         "Invalid floating point encoding");
   }
 
@@ -1143,15 +1175,27 @@ public:
   SPIRVTypeCooperativeMatrixKHR();
   _SPIRV_DCL_ENCDEC
   llvm::Optional<ExtensionID> getRequiredExtension() const override {
+    SPIRVType *Ty = this->getCompType();
+    if (Ty->isTypeInt() && static_cast<SPIRVTypeInt *>(Ty)->getBitWidth() == 4)
+      this->getModule()->addExtension(ExtensionID::SPV_INTEL_int4);
     return ExtensionID::SPV_KHR_cooperative_matrix;
   }
   SPIRVCapVec getRequiredCapability() const override {
     auto CV = getVec(CapabilityCooperativeMatrixKHR);
     if (CompType->isTypeFloat(16, FPEncodingBFloat16KHR))
       CV.push_back(CapabilityBFloat16CooperativeMatrixKHR);
+    else if (CompType->isTypeInt() &&
+             static_cast<SPIRVTypeInt *>(CompType)->getBitWidth() == 4)
+      CV.push_back(CapabilityInt4CooperativeMatrixINTEL);
+    else if (CompType->isTypeFloat(8, FPEncodingFloat8E4M3EXT) ||
+             CompType->isTypeFloat(8, FPEncodingFloat8E5M2EXT))
+      CV.push_back(CapabilityFloat8CooperativeMatrixEXT);
+    else if (CompType->isTypeFloat(4, internal::FPEncodingFloat4E2M1INTEL))
+      CV.push_back(internal::CapabilityFloat4E2M1CooperativeMatrixINTEL);
     return CV;
   }
 
+  std::vector<SPIRVValue *> getArgs() const { return Args; }
   SPIRVType *getCompType() const { return CompType; }
   SPIRVValue *getScope() const { return Args[0]; }
   SPIRVValue *getRows() const { return Args[1]; }
